@@ -1,6 +1,6 @@
 /* Util.h
  *
- * Copyright (c) 1996-2001 Mike Gleason, NCEMRSoft.
+ * Copyright (c) 1996-2005 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -84,22 +84,6 @@ int GetPwUid(struct passwd *pwp, const uid_t uid, char *const pwbuf, size_t pwbu
 int GetPwNam(struct passwd *pwp, const char *const nam, char *const pwbuf, size_t pwbufsize);
 #endif
 void CloseFile(FILE **);
-void PrintF(const FTPCIPtr cip, const char *const fmt, ...)
-#if (defined(__GNUC__)) && (__GNUC__ >= 2)
-__attribute__ ((format (printf, 2, 3)))
-#endif
-;
-void FTPLogError(const FTPCIPtr cip, const int pError, const char *const fmt, ...)
-#if (defined(__GNUC__)) && (__GNUC__ >= 2)
-__attribute__ ((format (printf, 3, 4)))
-#endif
-;
-#if defined(WIN32) || defined(_WINDOWS) || defined(__CYGWIN__)
-char *StrFindLocalPathDelim(const char *src);
-char *StrRFindLocalPathDelim(const char *src);
-void TVFSPathToLocalPath(char *dst);
-void LocalPathToTVFSPath(char *dst);
-#endif
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 int gettimeofday(struct timeval *const tp, void *junk);
 #endif
